/****************************************************************************
	Sunflower C library include file for Chickadee.
	Copyright 1995 BAGOTRONIX
	Modify at will but include copyright info.

	chickade.h
	This file defines constants and macros for the Chickadee.
****************************************************************************/

#ifndef CHICKADE_ALREADY_INCLUDED
#define CHICKADE_ALREADY_INCLUDED

#include <dos.h>

#include "stdinc.h"

#define CHICKADEE

#define IOB186  0xff00

/*
//      Define the system peripheral port addresses
*/
#define RELOC       IOB186 + 0xfe       /* Relocation register */

#define MPCS        IOB186 + 0xa8       /* Chip selection control registers */
#define MMCS        IOB186 + 0xa6
#define PACS        IOB186 + 0xa4
#define LMCS        IOB186 + 0xa2
#define UMCS        IOB186 + 0xa0

#define MDRAM       IOB186 + 0xe0       /* 80C186 extensions */
#define CDRAM       IOB186 + 0xe2
#define EDRAM       IOB186 + 0xe4

#define T0CNT       IOB186 + 0x50       /* Timer registers */
#define T0CMPA      IOB186 + 0x52
#define T0CMPB      IOB186 + 0x54
#define T0CON       IOB186 + 0x56
#define T1CT        IOB186 + 0x58
#define T1CTA       IOB186 + 0x5a
#define T1CTB       IOB186 + 0x5c
#define T1MD        IOB186 + 0x5e
#define T2CT        IOB186 + 0x60
#define T2CTA       IOB186 + 0x62
#define T2MD        IOB186 + 0x66

/* DMA registers */
#define D0SRCL    IOB186 + 0xc0
#define D0SRCH    IOB186 + 0xc2
#define D0DSTL    IOB186 + 0xc4
#define D0DSTH    IOB186 + 0xc6
#define D0TC      IOB186 + 0xc8
#define D0CON     IOB186 + 0xca
#define D1SRCL    IOB186 + 0xd0
#define D1SRCH    IOB186 + 0xd2
#define D1DSTL    IOB186 + 0xd4
#define D1DSTH    IOB186 + 0xd6
#define D1TC      IOB186 + 0xd8
#define D1CON     IOB186 + 0xda

/* Master mode interrupt controller port definitions */
#define EOI         IOB186 + 0x22       /* End of interrupt register */
#define POLL        IOB186 + 0x24       /* Poll register */
#define PSTAT       IOB186 + 0x26       /* Poll status register */
#define IMASK       IOB186 + 0x28       /* Interrupt mask register */
#define PRIMASK     IOB186 + 0x2a       /* Priority mask register */
#define INSER       IOB186 + 0x2c       /* In-service register */
#define IREQ        IOB186 + 0x2e       /* Interrupt request register */
#define ISTAT       IOB186 + 0x30       /* Interrupt status register */
#define TMRCTL      IOB186 + 0x32       /* Timer control register */
#define DMA0CON     IOB186 + 0x34       /* DMA 0 control register */
#define DMA1CON     IOB186 + 0x36       /* DMA 1 control register */
#define I0CON     IOB186 + 0x38       /* INT0 control register */
#define I1CON     IOB186 + 0x3a       /* INT1 control register */
#define I2CON     IOB186 + 0x3c       /* INT2 control register */
#define I3CON     IOB186 + 0x3e       /* INT3 control register */

#define PDCON       IOB186 + 0xf0       /* 80C188 power save control */

/*  Chickadee interrupts:
	0       divide error
	1       single step
	2       NMI (expansion bus, power monitor)
	3       breakpoint
	4       overflow
	5       array bounds
	6       unused opcode
	7       escape opcode
	8       TIMER 0
	9       reserved
	A       DMA0 (used as IRQ6 on PC/104 expansion bus)
	B       DMA1 (used as IRQ3 on PC/104 expansion bus)
	C       INT0 (used as IRQ4 on PC/104 expansion bus)
	D       INT1 (used as IRQ5 on PC/104 expansion bus)
	E       INT2 (used as IRQ2 on PC/104 expansion bus)
	F       INT3 (used as IRQ7 on PC/104 expansion bus)
	10      numerics (186 only)
	11      reserved
	12      TIMER 1
	13      TIMER 2
	FE      used by COMMkernenter() in TD-kernel
	FF      used by COMMkernexit() in TD-kernel
*/

#define INT3NUM 15
#define INT2NUM 14
#define INT1NUM 13
#define INT0NUM 12
#define DMA1NUM 11
#define DMA0NUM 10
#define T2NUM 19
#define T1NUM 18
#define T0NUM 8
#define NMINUM 2

#define IRQ2 INT2NUM
#define IRQ3 DMA1NUM
#define IRQ4 INT0NUM
#define IRQ5 INT1NUM
#define IRQ6 DMA0NUM
#define IRQ7 INT3NUM

#define COMA 0x400
#define COMB 0x480
#define LPT  0x500
#define LPTDATA     LPT
#define LPTSTATUS   LPT+1
#define LPTCONTROL  LPT+2
#define LPTBUSY     0x80
#define LPTACK      0x40
#define LPTPE       0x20
#define LPTSLCT     0x10
#define LPTERROR    0x08
#define LPTIRQENB   0x10
#define LPTSLIN     0x08
#define LPTINIT     0x04
#define LPTAFD      0x02
#define LPTSTB      0x01


#define RBR 0
#define THR 0
#define IER 1
#define IIR 2
#define LCR 3
#define MCR 4
#define LSR 5
#define MSR 6
#define SCR 7
#define DLL 0
#define DLM 1

#define B110    1047
#define B300    384
#define B600    192
#define B1200   96
#define B2400   48
#define B4800   24
#define B9600   12
#define B19200  6
#define B38400  3
#define B57600  2
#define B115200 1

#define CHICKADEE_8255	0x580
#define PIA_A 0x580
#define PIA_B 0x581
#define PIA_C 0x582
#define PIA_MODE 0x583
#define WD  0x700

#define SEOI(x) outport(EOI,x)          /* specific EOI */
#define NEOI() outport(EOI,0x8000)      /* non-specific EOI */

#define NOP() __emit__(0x90)

/* nmemonic labels for interrupt controller */
#define EDGE 0
#define LEVEL 1


#ifdef __cplusplus
	extern "C" {
#endif

void IRQinit (UBYTE, void interrupt (*)(), UBYTE, BOOLEAN);
void mask (int);
void unmask (int);
void CHKdelay (UWORD);

#ifdef __cplusplus
	}
#endif


/****************************************************************************
DLY():  macro to convert delay intervals into CHKdelay() iterations
	arguments:  the time to delay in ns
	returns:  the delay in CHKdelay() iterations (unsigned)
	example #1:
		{
		asm push cx;			{ if you need to save cx }
		_CX = DLY (4100000);    { 4.1 millisecond delay }
		CHKdelay();
		asm pop cx;				{ if you need to restore cx }
		}
	example #2:
		{
		somevariable = DLY (wastetime);    { variable delay }
		...some more code...
		asm push cx;			{ if you need to save cx }
		_CX = somevariable;
		CHKdelay();
		asm pop cx;				{ if you need to restore cx }
		}
	divisors:
		8  MHz -> divisor = 1875
		10 MHz -> divisor = 1500
		12 MHz -> divisor = 1250
		16 MHz -> divisor = 937
		20 MHz -> divisor = 750
		25 MHz -> divisor = 600
	verified:  7/10/96 8 MHz divisor only
****************************************************************************/
#define DLY(x)  (UWORD)((ULONG)x/(ULONG)1875+1L) /* add 1 to ensure at least 1 iteration */


#if 0
/****************************************************************************
CHKdelay():  delays 16 clocks per iteration (80C188)
	arguments:  place iteration count in cx
	returns:	nothing
	resources used:  cx
	comments:	16 clocks / iteration, 6 clocks for fall-thru
		Intel data book says 16 clocks / iteration, but I don't believe it.
		Profiling indicates that 15 clocks / iteration is more likely.
	CAUTION:  cx = 0 iterates 65536 times.  You should set cx >= 1.
		Timings are not exact.  Interrupts, wait states, refresh & DMA cycles
		can all butt in.  CHKdelay() should be thought of as delaying
		AT LEAST the # of clocks in cx.
	verified:  7/10/96
****************************************************************************/
#define CHKdelay() __emit__(0xe2, 0xfe);	/* asm equiv:  loop $-2 */
#endif


#endif

